package database

import (
	"context"
	"fmt"
	"log"
	"os"
	"strings"

	"git.paulbsd.com/paulbsd/ipbl/src/config"
	"git.paulbsd.com/paulbsd/ipbl/src/models"
	_ "github.com/lib/pq"
	"xorm.io/xorm"
	"xorm.io/xorm/dialects"
	"xorm.io/xorm/names"
)

func Initialize(ctx *context.Context, cfg *config.Config) (err error) {
	var databaseEngine = "postgres"
	//cacher := caches.NewLRUCacher(caches.NewMemoryStore(), 10)

	cfg.Db, err = xorm.NewEngine(databaseEngine,
		fmt.Sprintf("%s://%s:%s@%s/%s",
			databaseEngine,
			cfg.DbParams.DbUsername,
			cfg.DbParams.DbPassword,
			cfg.DbParams.DbHostname,
			cfg.DbParams.DbDatabase))
	if err != nil {
		log.Fatalln(err)
	}
	cfg.Db.SetMapper(names.GonicMapper{})
	cfg.Db.SetQuotePolicy(dialects.QuotePolicyReserved)
	cfg.Db.ShowSQL(cfg.Options.Debug)

	//cfg.Db.SetDefaultCacher(cacher)

	if cfg.Switchs.Drop {
		log.Println("Dropping tables")
		models.DropTables(ctx, cfg)
		os.Exit(0)
	}

	log.Println("Syncing tables")
	err = models.NewEngine(ctx, cfg)

	if cfg.Switchs.Migrate {
		migrate(cfg)
	}

	return
}

func migrate(cfg *config.Config) (err error) {
	var cfgsets []models.CfgSet
	num, err := cfg.Db.FindAndCount(&cfgsets)
	fmt.Printf("%d sets found\n", num)
	if err != nil {
		fmt.Println(err)
	}
	for _, cfgset := range cfgsets {
		var reg = cfgset.Regex
		reg = strings.Trim(reg, "()")
		rs := strings.Split(reg, "|")
		for _, v := range rs {
			if v != "" {
				n := models.CfgExpr{
					Expr:    v,
					Set:     &cfgset,
					Enabled: true,
				}
				cfg.Db.Insert(&n)
			}
		}
	}
	return
}
