package ws

import (
	"encoding/json"
	"log"

	"git.paulbsd.com/paulbsd/ipbl/src/config"
	"git.paulbsd.com/paulbsd/ipbl/src/models"
	"github.com/labstack/echo/v4"
	"golang.org/x/net/websocket"
)

// HandleWSRR handle req rep flows
func HandleWSRR(c *echo.Context, cfg *config.Config) error {
	websocket.Handler(func(ws *websocket.Conn) {
		defer ws.Close()

		var welcome = wsWelcome{}

		var msg []byte
		err := websocket.Message.Receive(ws, &msg)
		if err != nil {
			log.Println(err)
		}

		err = json.Unmarshal(msg, &welcome)
		if err == nil {
			welcomeAgents(ws, welcome, "rr")
		}

		var lastip string
		for {
			var apievent = models.APIEvent{}
			var event = models.Event{}

			var msg []byte
			err := websocket.Message.Receive(ws, &msg)
			if err != nil {
				log.Printf("disconnect: %s (from reqrep channel)", welcome.Hostname)
				return
			}

			err = json.Unmarshal(msg, &apievent)
			if err != nil {
				log.Println(err)
			}

			if lastip != "" && apievent.IPData.IP == lastip {
				continue
			}

			switch apievent.MsgType {
			case "bootstrap":
				log.Printf("bootstrap: %s\n", apievent.Hostname)
				listeners.Range(func(index, value interface{}) bool {
					if index != apievent.Hostname && value.(*connectionInfo).ConnectionPS != nil {
						err = websocket.Message.Send(value.(*connectionInfo).ConnectionPS, msg)
						if err != nil {
							log.Println(err)
							ws.Close()
							gcConnOnError(ws)
						}
					}
					return true
				})
			case "add":
				session := cfg.Db.NewSession()
				event.APIParse(session, apievent)
				session.Close()

				err := event.Insert(cfg)
				if err != nil {
					log.Println(err)
				}

				listeners.Range(func(index, value interface{}) bool {
					if value.(*connectionInfo).ConnectionPS != nil {
						err = websocket.Message.Send(value.(*connectionInfo).ConnectionPS, msg)
						if err != nil {
							log.Println(err)
							ws.Close()
							gcConnOnError(ws)
						}
					}
					return true
				})
				log.Printf("ws: Inserted event")
			case "init":
				listeners.Range(func(index, value interface{}) bool {
					if value.(*connectionInfo).ConnectionPS != nil {
						err = websocket.Message.Send(value.(*connectionInfo).ConnectionPS, msg)
						if err != nil {
							log.Println(err)
							ws.Close()
							gcConnOnError(ws)
						}
					}
					return true
				})
			case "ping":
				listeners.Range(func(index, value interface{}) bool {
					if index == apievent.Hostname && value.(*connectionInfo).ConnectionPS != nil {
						err = websocket.Message.Send(value.(*connectionInfo).ConnectionPS, msg)
						if err != nil {
							log.Println(err)
							ws.Close()
							gcConnOnError(ws)
						}
					}
					return true
				})
			default:
			}

			err = websocket.Message.Send(ws, "OK")
			if err != nil {
				log.Println(err)
				gcConnOnError(ws)
			}
		}
	}).ServeHTTP((*c).Response(), (*c).Request())
	return nil
}
